/*  osdchat - on-screen display messaging service
 *  Copyright (c) 2003 by Mike Smelov  <mike-at-argoncorp-dot-com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See LICENSE for details.
 *
 */

 /* osd.c */

#include "osdchatd.h"
#include "common.h"

/* defined in config.c */
extern struct xosd_s xset;
extern struct pref_s pref;

/* defined in xosd library */
extern char *xosd_error;

/* Initialize xosd and setup scroll buffer
   Return 1 if successful.
*/
int init_xosd(void)
{
	char *default_color = DEFAULT_XOSD_COLOR;
	char *default_font  = DEFAULT_XOSD_FONT;

	osd = xosd_create(xset.lines);

	if (osd == NULL) {
		printDBG(LVL_SYSLOG, "xosd_create(): %s", xosd_error);
		return 0;
	}

	xosd_set_pos(osd, xset.pos);
	xosd_set_vertical_offset(osd, xset.offset);
	xosd_set_timeout(osd, xset.delay);

	if (xosd_set_colour(osd, xset.color) == -1) {
		printDBG(LVL_SYSLOG, "Invalid X Color: choosing built-in default");
		if (xosd_set_colour(osd, default_color) == -1) {
			printDBG(LVL_SYSLOG, "Invalid X Color: rgb.txt probably missing");
			return 0;
		}
	}

	if (xosd_set_font(osd, xset.font) == -1) {
		printDBG(LVL_SYSLOG, "Invalid X Font: choosing built-in default");
		if (xosd_set_font(osd, default_font) == -1) {
			printDBG(LVL_SYSLOG, "Invalid X Font: misc font set probably missing");
			return 0;
		}
	}

	if (!osdbuf) {
		osdbuf = (char **) xmalloc(xset.lines * sizeof(char *));
	}

	return 1;
}

/* Show (timestamp), prompt and message on OSD.
   Return 1 if message filled buffer limit, so
   that client warning level can be adjusted.
*/
int show_osd(char *prompt, char *msg)
{
	int i, k, s;
	int inc, start, end;
	int prompt_len, next_len, time_len;
	char *tmp, *next;
	char tstamp[32];
	time_t timestamp;
	static int next_pos = -1;

	if (pref.timestamp) {
		timestamp = time(NULL);
		strftime(tstamp, 32, "%X", localtime(&timestamp));
	} else {
		tstamp[0] = NIL;
	}

	next = msg;
	next_len = strlen(next);
	prompt_len = strlen(prompt);
	time_len = strlen(tstamp);

	if (next_pos == -1) {
		next_pos = (pref.move == DOWN) ? 0 : xset.lines-1;
	}

	if (pref.move == DOWN) {
		s = 1;
		end = xset.lines - 1;
		start = 0;
	} else {
		s = -1;
		end = 0;
		start = xset.lines - 1;
	}

	inc = start;

	do {
		tmp = (char *) xmalloc(time_len + prompt_len + next_len + 3);

		if (next == msg) {
			if (pref.timestamp) {
				snprintf(tmp, time_len + prompt_len + 2, "%s %s", tstamp, prompt);
			} else {
				strncpy(tmp, prompt, prompt_len + 1);
			}
		} else {
			snprintf(tmp, time_len + prompt_len + 2, "%*s", time_len + prompt_len + (pref.timestamp ? 1 : 0), " ");
		}

		strncat(tmp, next, pref.wrapcol);
		osdbuf[inc] = tmp;

		log_msgs(tmp);

		next_len = strlen(next);
		next += pref.wrapcol;

		if (debug == 1) {
			printDBG(LVL_DEBUG, "next_pos: %d\tstart: %d\t inc: %d\n", next_pos, start, inc);
			for (i = start; i-s != next_pos; i+=s)
				printDBG(LVL_DEBUG, "osdbuf[%d]: %s %p\n", i, osdbuf[i], osdbuf[i]);
		}

		if (next_len <= pref.wrapcol) {
			for (i = start; i-s != next_pos; i+=s)
				xosd_display(osd, i, XOSD_string, osdbuf[i]);

			k = start;
		} else {
			k = inc;
		}

		if (next_pos != end) {
			next_pos += s;
		} else {
			free(osdbuf[end]);
		}

		/* scroll lines */
		for (i = next_pos; i != k; i-=s)
			osdbuf[i] = osdbuf[i-s];

		inc += (pref.move == DOWN) ? 1 : 0;
	}
	while (next_len > pref.wrapcol && inc-s != end);

	if (inc-s == end) {
		xosd_wait_until_no_display(osd);
		return 1;
	}

	return 0;
}
